"use strict";
/* eslint-env jest */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const models_1 = require("./models");
const koa_1 = __importDefault(require("koa"));
const koa_bodyparser_1 = __importDefault(require("koa-bodyparser"));
const mongodb_memory_server_1 = require("mongodb-memory-server");
const index_1 = __importStar(require("../src/index"));
const supertest_1 = __importDefault(require("supertest"));
const mongod = new mongodb_memory_server_1.MongoMemoryServer({
    autoStart: false
});
const setupServer = () => {
    const app = new koa_1.default();
    app.use(koa_bodyparser_1.default());
    return app;
};
let db;
describe('User Test', () => {
    beforeAll(() => __awaiter(this, void 0, void 0, function* () {
        yield mongod.start();
        const url = yield mongod.getConnectionString();
        db = yield models_1.connect(`${url}`);
    }));
    afterAll(() => __awaiter(this, void 0, void 0, function* () {
        // await disconnect()
        yield db.connection.close();
        yield mongod.stop();
    }));
    it('basic', () => __awaiter(this, void 0, void 0, function* () {
        const app = setupServer();
        const router = index_1.default('/users', models_1.User, [
            index_1.CREATE,
            index_1.GET_LIST,
            index_1.GET_ONE,
            index_1.UPDATE,
            index_1.DELETE
        ]);
        app.use(router.routes()).use(router.allowedMethods());
        const server = app.callback();
        // post
        let res = yield supertest_1.default(server)
            .post('/users')
            .send({ name: 'takashi' });
        expect(res.status).toBe(201);
        expect(res.body.id).not.toBeNull();
        expect(res.body.name).toBe('takashi');
        const { id } = res.body;
        // put
        res = yield supertest_1.default(server)
            .put(`/users/${id}`)
            .send({ name: 'toru' });
        expect(res.status).toBe(200);
        expect(res.body.name).toBe('toru');
        // get
        res = yield supertest_1.default(server).get(`/users/${id}`);
        expect(res.status).toBe(200);
        expect(res.body.name).toBe('toru');
        yield supertest_1.default(server)
            .post('/users')
            .send({ name: 'kaori' });
        yield supertest_1.default(server)
            .post('/users')
            .send({ name: 'bob' });
        yield supertest_1.default(server)
            .post('/users')
            .send({ name: 'steven' });
        // get list
        res = yield supertest_1.default(server)
            .get('/users')
            .query({
            sort: JSON.stringify(['name', 'ASC']),
            range: JSON.stringify([0, 2])
        });
        expect(res.status).toBe(200);
        expect(res.body).toHaveLength(2);
        const users = res.body;
        expect(users[0].name).toBe('bob');
        expect(users[1].name).toBe('kaori');
    }));
});
